@@ -1,20 +1,20 @@
-/*  Arg_parser - POSIX/GNU command line argument parser. (C++ version)
-    Copyright (C) 2006-2019 Antonio Diaz Diaz.
+/* Arg_parser - POSIX/GNU command line argument parser. (C++ version)
+   Copyright (C) 2006-2021 Antonio Diaz Diaz.
 
-    This library is free software. Redistribution and use in source and
-    binary forms, with or without modification, are permitted provided
-    that the following conditions are met:
+   This library is free software. Redistribution and use in source and
+   binary forms, with or without modification, are permitted provided
+   that the following conditions are met:
 
-    1. Redistributions of source code must retain the above copyright
-    notice, this list of conditions and the following disclaimer.
+   1. Redistributions of source code must retain the above copyright
+   notice, this list of conditions, and the following disclaimer.
 
-    2. Redistributions in binary form must reproduce the above copyright
-    notice, this list of conditions and the following disclaimer in the
-    documentation and/or other materials provided with the distribution.
+   2. Redistributions in binary form must reproduce the above copyright
+   notice, this list of conditions, and the following disclaimer in the
+   documentation and/or other materials provided with the distribution.
 
-    This library is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+   This library is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 */
 
 #include <cstring>
@@ -167,7 +167,7 @@ Arg_parser::Arg_parser( const int argc, const char * const argv[],
       else non_options.push_back( argv[argind++] );
       }
     }
-  if( error_.size() ) data.clear();
+  if( !error_.empty() ) data.clear();
   else
     {
     for( unsigned i = 0; i < non_options.size(); ++i )
@@ -190,7 +190,7 @@ Arg_parser::Arg_parser( const char * const opt, const char * const arg,
       { if( opt[2] ) parse_long_option( opt, arg, options, argind ); }
     else
       parse_short_option( opt, arg, options, argind );
-    if( error_.size() ) data.clear();
+    if( !error_.empty() ) data.clear();
     }
   else data.push_back( Record( opt ) );
   }
